#!/usr/bin/env python
#Copyright (C) 2007 - JamesWP
#Copyright (C) 2007 - Daniel Bessler <danielb@kiba-dock.og>

from commands import getoutput
from os import path
from time import sleep
from poplib import *
from sys import exit

import dbus
from dbus.mainloop.glib import DBusGMainLoop
import gobject

PLUGIN_NAME = "kiba-mail"

#Check every 5 minutes for new mails
CHECK_INTERVALL = 300000

# Should the icon blink on and off? And for how long? (Seconds)
BLINK = 'yes'
BLINK_DURATION = 30

## MAIL SERVER
VISUAL_NAME = 'Mail Account'

if getoutput("kiba-settings --get-bool mail_enable") != "TRUE":
	exit()

LAUNCHER = getoutput("gset-kiba --get-char mail_client")
if LAUNCHER == "Unable to find Option 'mail_client' in the Settings File":
	LAUNCHER = 'mozilla-thunderbird'
SERVER_NAME = getoutput("gset-kiba --get-char mail_server_name")
if SERVER_NAME == "Unable to find Option 'mail_server_name' in the Settings File":
	SERVER_NAME = ''
SERVER_TYPE = getoutput("gset-kiba --get-char mail_server_type")
if SERVER_TYPE == "Unable to find Option 'mail_server_type' in the Settings File":
	SERVER_TYPE = ''
USER_NAME = getoutput("gset-kiba --get-char mail_username")
if USER_NAME == "Unable to find Option 'mail_username' in the Settings File":
	USER_NAME = ''
PASSWORD = getoutput("gset-kiba --get-char mail_password")
if PASSWORD == "Unable to find Option 'mail_password' in the Settings File":
	PASSWORD = ''

#Set this if you want to use a custom Theme Directory
THEME_DIRECTORY = ""
if THEME_DIRECTORY == "":
	if path.isdir('/home/mandrake/rpm/BUILD/kiba-dbus-plugins/themes/' + PLUGIN_NAME) == 1:
		THEME_DIRECTORY = '/home/mandrake/rpm/BUILD/kiba-dbus-plugins/themes/' + PLUGIN_NAME
	elif path.isdir('/usr/share/kiba-dock/icons/' + PLUGIN_NAME) == 1:
		THEME_DIRECTORY = '/usr/share/kiba-dock/icons/' + PLUGIN_NAME
	else:
		print "Kiba-Mail: No theme found, exiting"
		exit()
THEME=THEME_DIRECTORY

if SERVER_NAME == "" or SERVER_TYPE == "" or USER_NAME == "" or PASSWORD == "":
	print "Kiba-Mail: Some Login Data is missing, exiting"
	exit()

class KibaMailPlugin(object):
	Messages=""
	Counter=0
	Detailed=''
	icon=THEME + '/none.png'
	icon_added=0
	blinks=0
	
	def __init__ (self):
		#start timeout, 1000 means every second
		source_id=gobject.timeout_add(CHECK_INTERVALL, self.timeout_func)
		
		#get the bus proxy
		self.bus = dbus.SessionBus()
		
		#Setup Signal Handlers (we need to avoid some uggly segfaults when kiba closes/reopenes)
		self.bus.add_signal_receiver(self.open_handler, dbus_interface = "org.kiba.dock.Kiba", signal_name = "Initialled")
		self.bus.add_signal_receiver(self.close_handler, dbus_interface = "org.kiba.dock.Kiba", signal_name = "Closed")
		self.bus.add_signal_receiver(self.context_menu_handler, dbus_interface = "org.kiba.dock.Kiba", signal_name = "IconMenu")
		self.bus.add_signal_receiver(self.button_release_handler, dbus_interface = "org.kiba.dock.Kiba", signal_name = "IconReleased")
		
		self.check_mails()
		
		#check if kiba is running on startup
		if getoutput("pidof kiba-dock") != "":
			self.add_kiba_icon()
			self.set_icon_and_titel()
	
	def check_mails (self):
		if SERVER_TYPE == 'POP3_SSL':
			self.Server = POP3_SSL(SERVER_NAME)
		else:
			self.Server = POP3(SERVER_NAME)
		
		# This sends the information to the POP3 Server
		print self.Server.getwelcome()
		print self.Server.user(USER_NAME)
		print self.Server.pass_(PASSWORD)
		
		self.Messages = self.Server.list()[1]
		self.Counter = (self.Counter + len(self.Messages))
		if len(self.Messages) > 0:
			self.Detailed = self.Detailed + ', ' + VISUAL_NAME + ': ' + str(len(self.Messages)) + ' New'
		
		#IMAP
		#imap = imaplib.IMAP4(SERVER_NAME)
		#imap.login(USER_NAME, USER_NAME)
		#typ, data = imap.select('Inbox', 1)
		#typ, data = imap.search(None, 'UNSEEN')
		#self.Counter = (self.Counter + len(data[0].split()))
		#imap.logout()
	
	def blink_icon(self):
		self.kiba.Pulse(PLUGIN_NAME)
		self.blinks+=1
		
		if self.blinks < BLINK_DURATION:
			return 1
		
		return 0
	
	def set_icon_and_titel(self):
		if self.Counter == 0:
			self.icon = THEME + '/none.png'
			self.kiba.SetTitel(PLUGIN_NAME, "You have %s New Emails%s" % (self.Counter, self.Detailed))
		if self.Counter == 1:
			self.icon = THEME + '/new.png'
			self.kiba.SetTitel(PLUGIN_NAME, "You have %s New Email%s" % (self.Counter, self.Detailed))
		if self.Counter > 1:
			self.icon = THEME + '/new.png'
			self.kiba.SetTitel(PLUGIN_NAME, "You have %s New Emails%s" % (self.Counter, self.Detailed))
		self.kiba.SetIcon(PLUGIN_NAME, self.icon)
		
		if self.Counter != 0:
			self.kiba.BubbleSetMessage(PLUGIN_NAME, "%s New Emails" % (self.Counter))
	
	def add_kiba_icon(self):
		#Initiall Kiba-DBus stuff
		self.obj = self.bus.get_object("org.kiba.dock.Kiba", "/org/kiba/dock/Kiba")
		self.kiba = dbus.Interface(self.obj, "org.kiba.dock.Kiba")
		
		#Add Icon to Kiba
		self.kiba.AddObject(PLUGIN_NAME, "NULL", self.icon)
		
		self.icon_added=1
		
	def context_menu_handler(self, id):
		if id != PLUGIN_NAME:
			return
	
	def button_release_handler(self, id):
		if id != PLUGIN_NAME:
			return
		
	def close_handler(self, sender=None):
		self.icon_added=0
	
	def open_handler(self, sender=None):
		self.add_kiba_icon()
	
	def timeout_func(self):
		if self.icon_added == 0:
			return 1
		
		self.check_mails()
		self.set_icon_and_titel()
		if self.Counter > 0 and Blink == 'yes':
			gobject.timeout_add(1000, self.blink_icon)
		
		return 1

#We need to use the Mainloop to catch the DBus Signals
DBusGMainLoop(set_as_default=True)
loop=gobject.MainLoop()

#Load the Plugin
kiba_mail=KibaMailPlugin()

#enter mainloop
loop.run()

if getoutput("pidof kiba-dock") != "":
	kiba_mail.kiba.RemoveObject(PLUGIN_NAME)
