#!/usr/bin/env python
#Copyright (C) 2007 - JamesWP
#Copyright (C) 2007 - Daniel Bessler <danielb@kiba-dock.og>

import commands
from time import sleep
from os import path
from sys import exit
from commands import getoutput

import dbus
from dbus.mainloop.glib import DBusGMainLoop
import gobject

PLUGIN_NAME = "kiba-signal"

# Intervall in milisec
CHECK_INTERVALL = 10000

if getoutput("kiba-settings --get-bool signal_enable") != "TRUE":
	exit()

INTERFACE = getoutput("kiba-settings --get-char signal_device")
if INTERFACE == "Unable to find Option 'signal_device' in the Settings File":
	INTERFACE = 'ra0'

#Set this if you want to use a custom Theme Directory
THEME_DIRECTORY = ""
if THEME_DIRECTORY == "":
	if path.isdir('/home/mandrake/rpm/BUILD/kiba-dbus-plugins/themes/' + PLUGIN_NAME) == 1:
		THEME_DIRECTORY = '/home/mandrake/rpm/BUILD/kiba-dbus-plugins/themes/' + PLUGIN_NAME
	elif path.isdir('/usr/share/kiba-dock/icons/' + PLUGIN_NAME) == 1:
		THEME_DIRECTORY = '/usr/share/kiba-dock/icons/' + PLUGIN_NAME
	else:
		print "Kiba-Signal: No theme found, exiting"
		exit()
THEME=THEME_DIRECTORY

COMMAND = 'iwconfig ' + INTERFACE

class KibaSignal(object):
	icon_added=0
	strength=0
	essid=''
	icon=THEME + '/0.png'
	
	def __init__ (self):
		#start timeout, 1000 means every second
		source_id=gobject.timeout_add(CHECK_INTERVALL, self.timeout_func)
		
		#get the bus proxy
		self.bus = dbus.SessionBus()
		
		#Setup Signal Handlers (we need to avoid some uggly segfaults when kiba closes/reopenes)
		self.bus.add_signal_receiver(self.open_handler, dbus_interface = "org.kiba.dock.Kiba", signal_name = "Initialled")
		self.bus.add_signal_receiver(self.close_handler, dbus_interface = "org.kiba.dock.Kiba", signal_name = "Closed")
		self.bus.add_signal_receiver(self.context_menu_handler, dbus_interface = "org.kiba.dock.Kiba", signal_name = "IconMenu")
		self.bus.add_signal_receiver(self.button_release_handler, dbus_interface = "org.kiba.dock.Kiba", signal_name = "IconReleased")
		
		self.get_signal()
		
		#check if kiba is running on startup
		if getoutput("pidof kiba-dock") != "":
			self.add_kiba_icon()
			self.set_icon_and_titel()
	
	# This part just checks through the output of the iwconfig command and finds out your signal strength
	def get_signal(self):
		output = str(commands.getoutput(COMMAND))
		#FIXME: this probably does not match all languages
		if output == "sh: iwconfig: command not found" or output == '':
			print "Kiba-Signal: No Output for WIFI device %s, exiting" % INTERFACE
			exit()
		
		output = output.split('Quality=')[1]
		output = output.split(' Signal')[0]
		signal = int(output.split('/')[0])
		maxsignal = int(output.split('/')[1])
		self.strength = int(100*(float(signal)/float(maxsignal)))
		output = str(commands.getoutput(COMMAND))
		output = output.split('ESSID:"')[1]
		self.essid = output.split('"')[0]
	
	def set_icon_and_titel(self):
		if self.strength == 0:
			self.icon = THEME + '/0.png'
		if self.strength > 0:
			self.icon = THEME + '/20.png'
		if self.strength > 19:
			self.icon = THEME + '/40.png'
		if self.strength > 39:
			self.icon = THEME + '/60.png'
		if self.strength > 59:
			self.icon = THEME + '/80.png'
		if self.strength > 79:
			self.icon = THEME + '/100.png'
		
		self.kiba.SetIcon(PLUGIN_NAME, self.icon)
		self.kiba.SetTitel(PLUGIN_NAME, '%s : %s%s' % (self.essid, self.strength, '%'))
	
	def add_kiba_icon(self):
		#Initiall Kiba-DBus stuff
		self.obj = self.bus.get_object("org.kiba.dock.Kiba", "/org/kiba/dock/Kiba")
		self.kiba = dbus.Interface(self.obj, "org.kiba.dock.Kiba")
		
		#Add Icon to Kiba
		self.kiba.AddObject(PLUGIN_NAME, "NULL", self.icon)
		
		self.icon_added=1
		
	def context_menu_handler(self, id):
		if id != PLUGIN_NAME:
			return
	
	def button_release_handler(self, id):
		if id != PLUGIN_NAME:
			return
		
	def close_handler(self, sender=None):
		self.icon_added=0
	
	def open_handler(self, sender=None):
		self.add_kiba_icon()
	
	def timeout_func(self):
		if self.icon_added == 0:
			return 1
		
		self.get_signal()
		self.set_icon_and_titel()
		
		return 1

#We need to use the Mainloop to catch the DBus Signals
DBusGMainLoop(set_as_default=True)
loop=gobject.MainLoop()

#Load the Plugin
kiba_signal=KibaSignal()

#enter mainloop
loop.run()

if getoutput("pidof kiba-dock") != "":
	kiba_signal.kiba.RemoveObject(PLUGIN_NAME)
